package cn.com.poc.knowledge.rest.impl;

import cn.com.poc.knowledge.aggregate.KnowledgeService;
import cn.com.poc.knowledge.dto.BizKnowledgeDocumentDto;
import cn.com.poc.knowledge.entity.BizKnowledgeDocumentEntity;
import cn.com.poc.knowledge.rest.KnowledgeRest;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class KnowledgeRestImpl implements KnowledgeRest {

    @Resource
    private KnowledgeService knowledgeService;

    @Override
    public List<BizKnowledgeDocumentDto> uploadDocument(MultipartFile[] documentFiles) throws Exception {
        Assert.notEmpty(documentFiles);
        Assert.isTrue(documentFiles.length <= 5, "Uploaded files can be no larger than five");
        List<BizKnowledgeDocumentDto> result = new ArrayList<>();
        List<BizKnowledgeDocumentEntity> entities = knowledgeService.uploadDocument(documentFiles);
        result = entities.stream().map(entity -> {
            BizKnowledgeDocumentDto dto = new BizKnowledgeDocumentDto();
            BeanUtil.copyProperties(entity, dto);
            return dto;
        }).collect(Collectors.toList());
        return result;
    }

    @Override
    public Boolean delDocument(Integer kdId) {
        return knowledgeService.delDocument(kdId);
    }

    @Override
    public Boolean renameDocument(Integer kdId, String rename) throws Exception {
        return knowledgeService.renameDocument(kdId, rename);
    }

    @Override
    public String enableKnowledge(Integer kdId) throws Exception {
        return knowledgeService.enableKnowledge(kdId);
    }

    @Override
    public Boolean trainKnowledge(List<Integer> kdIds) throws Exception {
        //执行任务
        knowledgeService.trainKnowledge(kdIds);
        return true;
    }

    @Override
    public List<BizKnowledgeDocumentDto> getList(String search,String trainStatus, PagingInfo pagingInfo) throws Exception {
        List<BizKnowledgeDocumentDto> result = new ArrayList<>();
        List<BizKnowledgeDocumentEntity> entities = knowledgeService.getList(search, trainStatus, pagingInfo);
        if (CollectionUtils.isNotEmpty(entities)) {
            result = entities.stream().map(entity -> {
                BizKnowledgeDocumentDto dto = new BizKnowledgeDocumentDto();
                BeanUtil.copyProperties(entity, dto);
                return dto;
            }).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public List<BizKnowledgeDocumentDto> getListByKdIds(List<Integer> kdIds) {
        List<BizKnowledgeDocumentDto> res = new ArrayList<>();
        Assert.notEmpty(kdIds);
        List<BizKnowledgeDocumentEntity> entities = knowledgeService.getListByKdIds(kdIds);
        if (CollectionUtils.isNotEmpty(entities)) {
            res = entities.stream().map(entity -> {
                BizKnowledgeDocumentDto dto = new BizKnowledgeDocumentDto();
                BeanUtil.copyProperties(entity, dto);
                return dto;
            }).collect(Collectors.toList());
        }
        return res;
    }
}
